//
//  QuickRearrangeTableView.m
//  QuickRearrangeTableView
//
//  Created by Sergey Pershenkov on 5/2/13.
//  Copyright (c) 2013 Sergey Pershenkov. All rights reserved.
//

#import "QuickRearrangeTableView.h"
#import <QuartzCore/QuartzCore.h>

@implementation QuickRearrangeTableView
{
  NSIndexPath *currentIndexPath_;
  UIImageView *draggingCellView_;
  NSTimer *scrollTimer_;
  CGFloat scrollDelta_;  
  BOOL cellCatched_;
  id savedObject_;
}

- (void)awakeFromNib
{
  [super awakeFromNib];
  
  [self addGestureRecognizer:[[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(longPress:)]];
  
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appBecomeActive) name:UIApplicationDidBecomeActiveNotification object:nil];
}

- (void)appBecomeActive
{
  [self performSelector:@selector(finishRearrange) withObject:nil afterDelay:0.3];
}

- (void)longPress:(UILongPressGestureRecognizer *)gestureRecognizer
{
  CGPoint locationInView = [gestureRecognizer locationInView:self];
  
  if(gestureRecognizer.state == UIGestureRecognizerStateBegan)
  {    
    currentIndexPath_ = [self indexPathForRowAtPoint:locationInView];
    
    if(currentIndexPath_)
    {
      cellCatched_ = YES;
      
      self.allowsSelection = NO;
      
      UITableViewCell *draggingCell = [self cellForRowAtIndexPath:currentIndexPath_];
      
      draggingCell.highlighted = NO;
      
      if(self.catchWithoutSeparator)
      {
        UIGraphicsBeginImageContextWithOptions(CGSizeMake(draggingCell.bounds.size.width, draggingCell.bounds.size.height - 1.0), YES, 0.0);
      }
      else
      {
        UIGraphicsBeginImageContextWithOptions(draggingCell.bounds.size, YES, 0.0);
      }
      
      [draggingCell.layer renderInContext:UIGraphicsGetCurrentContext()];
      
      draggingCellView_ = [[UIImageView alloc] initWithImage:UIGraphicsGetImageFromCurrentImageContext()];
      
      UIGraphicsEndImageContext();

      if(self.catchWithoutSeparator)
      {
        CGRect frame = [self rectForRowAtIndexPath:currentIndexPath_];
      
        frame.size.height -= 1.0;
      
        draggingCellView_.frame = frame;
      }
      else
      {
        draggingCellView_.frame = [self rectForRowAtIndexPath:currentIndexPath_];
      }
      
      [self addSubview:draggingCellView_];
      
      draggingCellView_.layer.shadowOffset = CGSizeMake(0.0, 0.0);
      
      draggingCellView_.layer.shadowRadius = 4.0;
      
      draggingCellView_.layer.shadowOpacity = 0.25;
      
      draggingCellView_.layer.shadowPath = [[UIBezierPath bezierPathWithRect:draggingCellView_.bounds] CGPath];
      
      [UIView animateWithDuration:0.2 animations:^
       {
         draggingCellView_.transform = CGAffineTransformMakeScale(1.05, 1.05);
         
         draggingCellView_.center = CGPointMake(160.0, locationInView.y);
       }
       completion:nil];

      scrollTimer_ = [NSTimer timerWithTimeInterval:0.001 target:self selector:@selector(scrollTableView:) userInfo:gestureRecognizer repeats:YES];
      
      [[NSRunLoop mainRunLoop] addTimer:scrollTimer_ forMode:NSDefaultRunLoopMode];
      
      savedObject_ = [self.delegate getObjectFromIndexPath:currentIndexPath_];
      
      [self reloadRowsAtIndexPaths:@[currentIndexPath_] withRowAnimation:UITableViewRowAnimationNone];
    }
  }
  else if(gestureRecognizer.state == UIGestureRecognizerStateChanged && cellCatched_)
  {    
    if(locationInView.y < self.bounds.origin.y)
    {
      locationInView.y = self.bounds.origin.y;
    }
    
    draggingCellView_.center = CGPointMake(160.0, locationInView.y);
    
    [self moveDummyRow:gestureRecognizer];
    
    if(self.contentSize.height > self.frame.size.height)
    {
      if(locationInView.y - self.bounds.origin.y > self.frame.size.height - draggingCellView_.frame.size.height/2.0)
      {
        if(locationInView.y - self.bounds.origin.y > self.frame.size.height - draggingCellView_.frame.size.height/3.0)
        {
          scrollDelta_ = 1.0;
        }
        else
        {
          scrollDelta_ = 0.5;
        }
      }
      else if(locationInView.y - self.bounds.origin.y < draggingCellView_.frame.size.height/2.0)
      {
        if(locationInView.y - self.bounds.origin.y < draggingCellView_.frame.size.height/3.0)
        {
          scrollDelta_ = -1.0;
        }
        else
        {
          scrollDelta_ = -0.5;
        }
      }
      else
      {
        scrollDelta_ = 0.0; 
      }
    }
    else
    {
      scrollDelta_ = 0.0;
    }
  }
  else if(gestureRecognizer.state == UIGestureRecognizerStateEnded)
  {
    [self finishRearrange];
  }
}

- (void)moveDummyRow:(UILongPressGestureRecognizer *)gestureRecognizer
{
  NSIndexPath *newIndexPath = [self indexPathForRowAtPoint:[gestureRecognizer locationInView:self]];
  
  if(newIndexPath && ![newIndexPath isEqual:currentIndexPath_])
  {
    [self.delegate moveRowFromIndexPath:currentIndexPath_ toIndexPath:newIndexPath];
    
    [self beginUpdates];
    
    [self deleteRowsAtIndexPaths:@[currentIndexPath_] withRowAnimation:UITableViewRowAnimationAutomatic];
    
    [self insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
    
    [self endUpdates];
    
    currentIndexPath_ = newIndexPath;
  }
}

- (void)scrollTableView:(NSTimer *)timer
{
  if(scrollDelta_ != 0.0)
  {   
    CGFloat newContentOffsetY = self.contentOffset.y + scrollDelta_;
    
    CGFloat bottomBorder = self.contentSize.height - self.frame.size.height;
    
    if(newContentOffsetY < 0.0)
    {
      [self setContentOffset:CGPointMake(0.0, 0.0) animated:NO];
    }
    else if(newContentOffsetY > bottomBorder)
    {
      [self setContentOffset:CGPointMake(0.0, bottomBorder) animated:NO];
    }
    else
    {
      [self setContentOffset:CGPointMake(0.0, newContentOffsetY) animated:NO];
    }
    
    CGFloat locationInViewY = [timer.userInfo locationInView:self].y;
    
    CGFloat selfBoundsOriginY = self.bounds.origin.y;
    
    if(locationInViewY < selfBoundsOriginY)
    {
      locationInViewY = selfBoundsOriginY;
    }
    
    draggingCellView_.center = CGPointMake(160.0, locationInViewY);
    
    [self moveDummyRow:timer.userInfo];
  }
}

- (void)finishRearrange
{
  if(cellCatched_)
  {
    [self.delegate setObject:savedObject_ atIndexPath:currentIndexPath_];
    
    [scrollTimer_ invalidate];
    
    scrollDelta_ = 0.0;
    
    cellCatched_ = NO;
    
    self.allowsSelection = YES;
    
    [UIView animateWithDuration:0.2 animations:^
     {
       draggingCellView_.transform = CGAffineTransformIdentity;
       
       draggingCellView_.frame = CGRectOffset(draggingCellView_.bounds, 0.0, [self rectForRowAtIndexPath:currentIndexPath_].origin.y);
     }
     completion:^(BOOL finished)
     {
       [self reloadRowsAtIndexPaths:@[currentIndexPath_] withRowAnimation:UITableViewRowAnimationNone];
       
       if(self.cellsAreMutable)
       {
         draggingCellView_.layer.shadowOpacity = 0.0;
         
         [self reloadSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationFade];
         
         [UIView animateWithDuration:0.2 animations:^
          {
            draggingCellView_.layer.opacity = 0.0;
          }
          completion:^(BOOL finished)
          {
            [draggingCellView_ removeFromSuperview];
          }];
       }
       else
       {
         [draggingCellView_ removeFromSuperview];
       }
     }];
  }
}

@end
